<?php

namespace SuperRhino\AppStoreServerApi\Response;

use Carbon\Carbon;
use SuperRhino\AppStoreServerApi\Enum\EnvironmentEnum;

class JWSTransaction extends AbstractResponse implements EnvironmentInterface {

    /**
     * A UUID that associates the transaction with a user on your own service. If your app doesn’t provide an appAccountToken, this string is empty.
     */
    public string $appAccountToken;

    /**
     * The bundle identifier of the app.
     */
    public string $bundleId;

    /**
     * The server environment, either sandbox or production.
     */
    public string $environment;

    /**
     * The UNIX time, in milliseconds, the subscription expires or renews.
     */
    public Carbon $expiresDate;

    /**
     * A string that describes whether the transaction was purchased by the user, or is available to them through Family Sharing.
     * "FAMILY_SHARED": The transaction belongs to a family member who benefits from the service.
     * "PURCHASED": The transaction belongs to the purchaser.
     * @see \SuperRhino\AppStoreServerApi\Enum\OwnerShipTypeEnum
     */
    public string $inAppOwnershipType;

    /**
     * A Boolean value that indicates whether the user upgraded to another subscription.
     */
    public bool $isUpgraded;

    /**
     * The offer code or the promotional offer identifier.
     */
    public string $offerIdentifier;

    /**
     * A value that represents the promotional offer type.
     * 1: An introductory offer.
     * 2: A promotional offer.
     * 3: An offer with a subscription offer code.
     * The offer types 2 and 3 have an offerIdentifier.
     */
    public int $offerType;

    /**
     * The purchase date of the transaction associated with the original transaction identifier.
     */
    public Carbon $originalPurchaseDate;

    /**
     * The transaction identifier of the original purchase.
     */
    public string $originalTransactionId;

    /**
     * The product identifier of the in-app purchase.
     */
    public string $productId;

    /**
     * The UNIX time, in milliseconds, that the App Store charged the user’s account for a purchase, restored product, subscription, or subscription renewal after a lapse.
     */
    public Carbon $purchaseDate;

    /**
     * The number of consumable products the user purchased.
     */
    public int $quantity;

    /**
     * The UNIX time, in milliseconds, that the App Store refunded the transaction or revoked it from Family Sharing.
     */
    public Carbon $revocationDate;

    /**
     * The reason that the App Store refunded the transaction or revoked it from Family Sharing.
     * 0:Apple Support refunded the transaction on behalf of the customer for other reasons; for example, an accidental purchase.
     * 1:Apple Support refunded the transaction on behalf of the customer due to an actual or perceived issue within your app.
     */
    public int $revocationReason;

    /**
     * The identifier of the subscription group the subscription belongs to.
     */
    public string $subscriptionGroupIdentifier;

    /**
     * The unique identifier of the transaction.
     */
    public string $transactionId;

    /**
     * The type of the in-app purchase.
     * "Auto-Renewable Subscription" : An auto-renewable subscription.
     * "Non-Consumable": A non-consumable in-app purchase.
     * "Consumable": A consumable in-app purchase.
     * "Non-Renewing Subscription": A non-renewing subcription.
     * @see \SuperRhino\AppStoreServerApi\Enum\ProductTypeEnum
     */
    public string $type;

    /**
     * The unique identifier of subscription purchase events across devices, including subscription renewals.
     * This value applies only to auto-renewable subscriptions.
     */
    public string $webOrderLineItemId;

    /**
     * The App Store signed the JSON Web Signature data.
     */
    public Carbon $signedDate;

    public function parameterConverter($name, $value) {
        switch ($name) {
            case 'expiresDate':
            case 'originalPurchaseDate':
            case 'purchaseDate':
            case 'revocationDate':
            case 'signedDate':
                return Carbon::createFromTimestampMs($value);
            default:
                return $value;
        }
    }

    public function isSandbox(): bool {
        return $this->environment == EnvironmentEnum::SANDBOX;
    }

    public function isProduction(): bool {
        return $this->environment == EnvironmentEnum::PRODUCTION;
    }

}